@@ -56,20 +56,12 @@ public:
   void collectGroup(unsigned) {}
   void collectVect(unsigned) {}
   void collectOtherList() {}
-  void collectCubicBezier(double, double, double, double, double, double) {}
-  void collectQuadraticBezier(double, double, double, double) {}
-  void collectMoveTo(double, double) {}
-  void collectLineTo(double, double) {}
-  void collectArcTo(double, double, bool, bool, double, double) {}
-  void collectClosePath() {}
+  void collectPath(const CDRPath &) {}
   void collectLevel(unsigned) {}
   void collectTransform(const CDRTransforms &, bool) {}
-  void collectFildId(unsigned) {}
-  void collectOutlId(unsigned) {}
-  void collectFild(unsigned id, unsigned short fillType, const CDRColor &color1, const CDRColor &color2, const CDRGradient &gradient, const CDRImageFill &imageFill);
-  void collectOutl(unsigned id, unsigned short lineType, unsigned short capsType, unsigned short joinType, double lineWidth,
-                   double stretch, double angle, const CDRColor &color, const std::vector<unsigned> &dashArray,
-                   unsigned startMarkerId, unsigned endMarkerId);
+  void collectFillStyle(unsigned short, const CDRColor &, const CDRColor &, const CDRGradient &, const CDRImageFill &) {}
+  void collectLineStyle(unsigned short, unsigned short, unsigned short, double, double, double, const CDRColor &,
+                        const std::vector<unsigned> &, const CDRPath &, const CDRPath &) {}
   void collectRotate(double,double,double) {}
   void collectFlags(unsigned, bool) {}
   void collectPageSize(double width, double height, double offsetX, double offsetY);
@@ -88,17 +80,18 @@ public:
   void collectSpnd(unsigned) {}
   void collectVectorPattern(unsigned, const WPXBinaryData &) {}
   void collectPaletteEntry(unsigned colorId, unsigned userId, const CDRColor &color);
-  void collectFont(unsigned fontId, unsigned short fontEncoding, const WPXString &font);
   void collectText(unsigned textId, unsigned styleId, const std::vector<unsigned char> &data,
-                   const std::vector<uint64_t> &charDescriptions, const std::map<unsigned, CDRCharacterStyle> &styleOverrides);
-  void collectArtisticText() {}
-  void collectParagraphText() {}
-  void collectStlt(const std::map<unsigned, CDRCharacterStyle> &charStyles);
+                   const std::vector<unsigned char> &charDescriptions, const std::map<unsigned, CDRCharacterStyle> &styleOverrides);
+  void collectArtisticText(double, double) {}
+  void collectParagraphText(double, double, double, double) {}
+  void collectStld(unsigned id, const CDRCharacterStyle &charStyle);
 
 private:
   CDRStylesCollector(const CDRStylesCollector &);
   CDRStylesCollector &operator=(const CDRStylesCollector &);
 
+  void getRecursedStyle(CDRCharacterStyle &charStyle, unsigned styleId);
+
   CDRParserState &m_ps;
   CDRPage m_page;
   std::map<unsigned, CDRCharacterStyle> m_charStyles;
