@@ -34,6 +34,7 @@
 #include <unicode/ucsdet.h>
 #include <unicode/ucnv.h>
 #include <unicode/utypes.h>
+#include <unicode/utf8.h>
 
 #define CDR_NUM_ELEMENTS(array) sizeof(array)/sizeof(array[0])
 
@@ -114,11 +115,10 @@ static unsigned short getEncodingFromICUName(const char *name)
 
 static unsigned short getEncoding(const unsigned char *buffer, unsigned bufferLength)
 {
-  if (!buffer || !bufferLength)
+  if (!buffer)
     return 0;
   UErrorCode status = U_ZERO_ERROR;
   UCharsetDetector *csd = 0;
-  const UCharsetMatch *csm = 0;
   try
   {
     csd = ucsdet_open(&status);
@@ -128,14 +128,28 @@ static unsigned short getEncoding(const unsigned char *buffer, unsigned bufferLe
     ucsdet_setText(csd, (const char *)buffer, bufferLength, &status);
     if (U_FAILURE(status))
       throw libcdr::EncodingException();
-    csm = ucsdet_detect(csd, &status);
+    const UCharsetMatch *csm = ucsdet_detect(csd, &status);
     if (U_FAILURE(status) || !csm)
       throw libcdr::EncodingException();
     const char *name = ucsdet_getName(csm, &status);
     if (U_FAILURE(status) || !name)
       throw libcdr::EncodingException();
+    int32_t confidence = ucsdet_getConfidence(csm, &status);
+    if (U_FAILURE(status))
+      throw libcdr::EncodingException();
+    CDR_DEBUG_MSG(("UCSDET: getEncoding name %s, confidence %i\n", name, confidence));
     unsigned short encoding = getEncodingFromICUName(name);
     ucsdet_close(csd);
+    /* From ICU documentation
+     * A confidence value of ten does have a general meaning - it is used
+     * for charsets that can represent the input data, but for which there
+     * is no other indication that suggests that the charset is the correct
+     * one. Pure 7 bit ASCII data, for example, is compatible with a great
+     * many charsets, most of which will appear as possible matches with
+     * a confidence of 10.
+     */
+    if (confidence == 10)
+      return 0;
     return encoding;
   }
   catch (const libcdr::EncodingException &)
@@ -145,57 +159,19 @@ static unsigned short getEncoding(const unsigned char *buffer, unsigned bufferLe
   }
 }
 
-static void _appendUCS4(WPXString &text, unsigned ucs4Character)
+static void _appendUCS4(WPXString &text, UChar32 ucs4Character)
 {
   // Convert carriage returns to new line characters
   // Writerperfect/LibreOffice will replace them by <text:line-break>
-  if (ucs4Character == 0x0d)
-    ucs4Character = (unsigned) '\n';
+  if (ucs4Character == (UChar32) 0x0d)
+    ucs4Character = (UChar32) '\n';
 
-  unsigned char first;
-  int len;
-  if (ucs4Character < 0x80)
-  {
-    first = 0;
-    len = 1;
-  }
-  else if (ucs4Character < 0x800)
-  {
-    first = 0xc0;
-    len = 2;
-  }
-  else if (ucs4Character < 0x10000)
-  {
-    first = 0xe0;
-    len = 3;
-  }
-  else if (ucs4Character < 0x200000)
-  {
-    first = 0xf0;
-    len = 4;
-  }
-  else if (ucs4Character < 0x4000000)
-  {
-    first = 0xf8;
-    len = 5;
-  }
-  else
-  {
-    first = 0xfc;
-    len = 6;
-  }
+  unsigned char outbuf[U8_MAX_LENGTH+1];
+  int i = 0;
+  U8_APPEND_UNSAFE(&outbuf[0], i, ucs4Character);
+  outbuf[i] = 0;
 
-  unsigned char outbuf[6] = { 0, 0, 0, 0, 0, 0 };
-  int i;
-  for (i = len - 1; i > 0; --i)
-  {
-    outbuf[i] = (ucs4Character & 0x3f) | 0x80;
-    ucs4Character >>= 6;
-  }
-  outbuf[0] = (ucs4Character & 0xff) | first;
-
-  for (i = 0; i < len; i++)
-    text.append(outbuf[i]);
+  text.append((char *)outbuf);
 }
 
 } // anonymous namespace
@@ -335,7 +311,7 @@ void libcdr::appendCharacters(WPXString &text, std::vector<unsigned char> charac
 {
   if (characters.empty())
     return;
-  static const unsigned short symbolmap [] =
+  static const UChar32 symbolmap [] =
   {
     0x0020, 0x0021, 0x2200, 0x0023, 0x2203, 0x0025, 0x0026, 0x220D, // 0x20 ..
     0x0028, 0x0029, 0x2217, 0x002B, 0x002C, 0x2212, 0x002E, 0x002F,
@@ -400,6 +376,7 @@ void libcdr::appendCharacters(WPXString &text, std::vector<unsigned char> charac
       break;
     case 0x88: // CHINESEBIG5
       conv = ucnv_open("windows-950", &status);
+      break;
     case 0xa1: // GREEEK
       conv = ucnv_open("windows-1253", &status);
       break;
@@ -437,72 +414,37 @@ void libcdr::appendCharacters(WPXString &text, std::vector<unsigned char> charac
       const char *srcLimit = (const char *)src + characters.size();
       while (src < srcLimit)
       {
-        uint32_t ucs4Character = (uint32_t)ucnv_getNextUChar(conv, &src, srcLimit, &status);
-        if (U_SUCCESS(status))
-        {
+        UChar32 ucs4Character = ucnv_getNextUChar(conv, &src, srcLimit, &status);
+        if (U_SUCCESS(status) && U_IS_UNICODE_CHAR(ucs4Character))
           _appendUCS4(text, ucs4Character);
-        }
       }
     }
     if (conv)
-    {
       ucnv_close(conv);
-    }
   }
 }
 
 void libcdr::appendCharacters(WPXString &text, std::vector<unsigned char> characters)
 {
-  for (std::vector<unsigned char>::const_iterator iter = characters.begin();
-       iter != characters.end();)
-  {
-    uint16_t high_surrogate = 0;
-    bool fail = false;
-    uint32_t ucs4Character = 0;
-    while (true)
-    {
-      if (iter == characters.end())
-      {
-        fail = true;
-        break;
-      }
-      uint16_t character = *iter++;
-      character |= (uint16_t)(*iter++) << 8;
-      if (character >= 0xdc00 && character < 0xe000) /* low surrogate */
-      {
-        if (high_surrogate)
-        {
-          ucs4Character = SURROGATE_VALUE(high_surrogate, character);
-          high_surrogate = 0;
-          break;
-        }
-        else
-        {
-          fail = true;
-          break;
-        }
-      }
-      else
-      {
-        if (high_surrogate)
-        {
-          fail = true;
-          break;
-        }
-        if (character >= 0xd800 && character < 0xdc00) /* high surrogate */
-          high_surrogate = character;
-        else
-        {
-          ucs4Character = character;
-          break;
-        }
-      }
-    }
-    if (fail)
-      throw libcdr::GenericException();
+  if (characters.empty())
+    return;
 
-    _appendUCS4(text, ucs4Character);
+  UErrorCode status = U_ZERO_ERROR;
+  UConverter *conv = ucnv_open("UTF-16LE", &status);
+
+  if (U_SUCCESS(status) && conv)
+  {
+    const char *src = (const char *)&characters[0];
+    const char *srcLimit = (const char *)src + characters.size();
+    while (src < srcLimit)
+    {
+      UChar32 ucs4Character = ucnv_getNextUChar(conv, &src, srcLimit, &status);
+      if (U_SUCCESS(status) && U_IS_UNICODE_CHAR(ucs4Character))
+        _appendUCS4(text, ucs4Character);
+    }
   }
+  if (conv)
+    ucnv_close(conv);
 }
 
 #ifdef DEBUG
